/* 
 * File:     PaeSimulator.c
 * Author:   MJW
 * Date:     23.03.2006
 * Modified: 23.03.2006
 *
 * Purpose:  Sourcecode fuer den Simulator
 */

#include "PaeSimulator.h"
#include "wrapper.h"

using namespace std;



PaeSimulator::PaeSimulator(void){

  polygonMode = GL_FILL;

  light_position[0] = 49.0;
  light_position[1] = 49.0;
  light_position[2] = 49.0;
  light_position[3] =  0.0;

  light_ambient[0] = 0.2;
  light_ambient[1] = 0.2;
  light_ambient[2] = 0.2;
  light_ambient[3] = 1.0;

  light_diffuse[0] = 1.0;
  light_diffuse[1] = 1.0;
  light_diffuse[2] = 0.7;
  light_diffuse[3] = 1.0;

  light_specular[0] = 1.0;
  light_specular[1] = 1.0;
  light_specular[2] = 1.0;
  light_specular[3] = 1.0;

  win_w = 500;
  win_h = 500;

  zoom = 1.0;

  flat = 0;

  cam = new PaeCamera();
  wor = new World();

}


void PaeSimulator::Display(void) {

    /* FrameBuffer und Z-Buffer loeschen */
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);


	glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

  	  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
      /* ************************************** *\
       * /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ *
       *   Hier die Zeichenbefehle einfuegen!!  *
      \* ************************************** */
      wor->Draw();
    glPopMatrix();

    glutSwapBuffers();
}




void PaeSimulator::Reshape(int width, int height) {

  win_w = width;
  win_h = height;
  
  /* Projection Transformation */
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  if (width > height) {
	glFrustum(-(float)width*zoom/(float)height,(float)width*zoom/(float)height,
			  -1.0*zoom,1.0*zoom,
			  1.0,100.0);
  } else {
	glFrustum(-1.0*zoom,1.0*zoom,
			  -(float)height*zoom/(float)width,(float)height*zoom/(float)width,
			  1.0,100.0);
  }

  glViewport(0, 0, (GLint)width, (GLint)height);
	
  /* Viewing Transformation */
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity ();
  cam->SetCam();

  glutPostRedisplay();
}


void PaeSimulator::Init(int argc, char **argv) {

  /*
   * Setting up the window
   */

  glutInit(&argc, argv);
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(win_w,win_h);
  glutInitDisplayMode(GLUT_DEPTH | GLUT_RGBA | GLUT_DOUBLE);
  
  if(glutCreateWindow("Digitale Schafweide") == GL_FALSE){
	exit(1);
  }

  /* alpha-testing */
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);  
     
  /* z-Buffering */
  glEnable(GL_DEPTH_TEST);

  /* Color => Material */
  glEnable(GL_COLOR_MATERIAL); 

  /* Normals are normalized automatically */
  glEnable(GL_NORMALIZE);

  /* Set light properties */ 
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);		

  /* Enable lighting computation and light0 */  
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  Reshape(win_w,win_h);

}





void PaeSimulator::Key(unsigned char key, int x, int y) {

  /* Switch lighting calculation on/off */
  switch(key) {

  case 'w':
    wor->terr->SwitchWater();
    break;

  case 'm':
    wor->terr->AddHeight(0.1);
    break;
  case 'M':
    wor->terr->AddHeight(-0.1);
    break;

  case 'h':
    cam->RotateDir(0.1);
    cam->SetCam();
    break;
  case 'H':
    cam->RotateDir(-0.1);
    cam->SetCam();
    break;

  case 't':
    cam->Rotate(0.1);
    cam->SetCam();
    break;
  case 'T':
    cam->Rotate(-0.1);
    cam->SetCam();
    break;

  case 'p':
    cam->Pitch(0.1);
    cam->SetCam();
    break;
  case 'P':
    cam->Pitch(-0.1);
    cam->SetCam();
    break;

  case 'y':
    cam->Yaw(0.1);
    cam->SetCam();
    break;
  case 'Y':
    cam->Yaw(-0.1);
    cam->SetCam();
    break;

  case 'r':
    cam->Roll(0.1);
    cam->SetCam();
    break;
  case 'R':
    cam->Roll(-0.1);
    cam->SetCam();
    break;

  case 's':
    wor->terr->SoftenTerrain();
    wor->terr->ReBuildNormals();
    break;

  case 'o':
    cam->StatusPrint();
    break;

  case 'l':
	if(glIsEnabled(GL_LIGHTING)) {
	  glDisable(GL_LIGHTING);
	} else {
	  glEnable(GL_LIGHTING);
	}
	glutPostRedisplay ();
    break;

  case 27:
  case 'q':
	exit(1);

  case 'f':
    flat = 1-flat;
	cout << endl << flat;
	if (flat > 0){
	  glShadeModel(GL_FLAT);
	} else {
	  glShadeModel(GL_SMOOTH);
    }
	glutPostRedisplay ();
	break;

  case ' ':
	if(polygonMode==GL_FILL){
	  polygonMode=GL_LINE;
    } else {
	  polygonMode=GL_FILL;
    }
    glPolygonMode (GL_FRONT_AND_BACK, polygonMode);

	glutPostRedisplay ();
	break;

  default:
    break;
  }
}


void PaeSimulator::SpecialKey(int key, int x, int y) {

  /* Model rotation via arrow keys */
  switch(key) {
  case GLUT_KEY_UP:
    cam->MoveUp(1.0);
	break;
  case GLUT_KEY_DOWN:
    cam->MoveUp(-1.0);
	break;
  case GLUT_KEY_LEFT:
    cam->MoveSide(1.0);
	break;
  case GLUT_KEY_RIGHT:
    cam->MoveSide(-1.0);
	break;
  case GLUT_KEY_PAGE_DOWN:
    cam->MoveForward(1.0);
    break;
  case GLUT_KEY_PAGE_UP:
    cam->MoveForward(-1.0);
    break;  
  default:
    break;
  }

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  cam->SetCam();
  glutPostRedisplay();
}


void PaeSimulator::Timer(int value){

  wor->Prepare();
  
  wor->Action();

  glutPostRedisplay();
  glutTimerFunc(50, WrapTimer, value+1);
}

